import requests, pymongo, re, logging, time
from bs4 import BeautifulSoup
import multiprocessing as mp

#生成日志
logging.basicConfig(filename='AuditCrawler.log', level='DEBUG')
#连接数据库
client = pymongo.MongoClient('mongodb://localhost:27017')
unit = client['AuditUnit']
auditors = client['Auditors']
#用以存储事务所的特征码
index = unit['index']
#存储爬取失败的事务所，后期调整
failed = unit['failed']
#用以存储事务所的信息
info = unit['info']
#用以存储事务所里的注册会计师
cpa = unit['CPA']
#用以存储事务所里的从业人员
NwPer = unit['NwPer']
#用以存储注册会计师详细资料
profile = auditors['profile']
#浏览器请求头
headers = {
    'User-Agent':'Mozilla/5.0(WindowsNT10.0;WOW64;Trident/7.0;rv:11.0)likeGecko'
}

#获取事务所信息
#获取事务所详情页面
def get_unit_page(token):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/09/'+ token + '.shtml'
    r = requests.post(url, headers= headers)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    return text

#从详情页面抽取信息存入数据库
def get_unit_info(token):
    text = get_unit_page(token)
    soup = BeautifulSoup(text, 'lxml')
    td = soup.find('table', "detail_table")
    tdl = td.find_all('td', "tdl")
    tdc = td.find_all('td', "data_tb_content")
    Unit_piece = {tdl[i].get_text('\n', '<br/>').replace('\n','').replace('\t', '').replace('（请点击）', ''): tdc[i].get_text('\n', '<br/>').replace('\n','').replace('\t', '').replace('（请点击）', '') for i in range(len(tdl))}
    info.insert_one(Unit_piece)


#获取从业人员信息
#获取总页数
def get_NwPer_num(token):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/OfficeIndexAction.do'
    params = {
       'method': 'getEmployeeList',
       'offGuid': token,
       'pageNum': 1,
       'pageSize': 10
    }
    r = requests.post(url, headers= headers, params=params)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    try:
        page_num = int(re.findall('记录 / 共 (.*?) 页', text)[0])
    except:
        page_num = 0
    return page_num

#获取从业人员列表页面
def get_NwPer_page(token, page):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/OfficeIndexAction.do'
    params = {
       'method': 'getEmployeeList',
       'offGuid': token,
       'pageNum': page,
       'pageSize': 10
    }
    r = requests.post(url, headers= headers, params=params)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    return text

#从列表页面抽取信息存入数据库
def get_NwPer_info(token):
    page_num = get_NwPer_num(token)
    if page_num==0:
        return 0
    for page in range(1, page_num+1):
        text = get_NwPer_page(token, page)
        soup = BeautifulSoup(text, 'lxml')
        td = soup.find('table', "detail_table").find_all('tr')
        if len(td) < 2:
            return 0
        unit = td[0].td.get_text('\n', '<br/>').replace('\n','').replace('\t', '').replace('（从业人员）', '')
        tdl = td[1].find_all('th')
        trcs = td[2:]
        for trc in trcs:
            tdc = trc.find_all('td')
            NwPer_piece = {tdl[i].get_text('\n', '<br/>').replace('\n','').replace('\t', ''): tdc[i].get_text('\n', '<br/>').replace('\n','').replace('\t', '') for i in range(len(tdl))}
            NwPer_piece['所在事务所'] = unit
            NwPer.insert_one(NwPer_piece)


#获取注册会计师信息
#获取总页数
def get_cpa_num(token):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/OfficeIndexAction.do'
    params = {
       'method': 'getPersons',
       'offGuid': token,
       'pageNum': 1,
       'pageSize': 10
    }
    r = requests.post(url, headers= headers, params=params)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    try:
        page_num = int(re.findall('记录 / 共 (.*?) 页', text)[0])
    except:
        page_num = 0
    return page_num

#获取注册会计师列表页面
def get_cpa_page(token, page):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/public/query/swszs/%20/'+ token +'.html'
    params = {
       'method': 'getPersons',
       'offGuid': token,
       'pageNum': page,
       'pageSize': 10
    }
    r = requests.post(url, headers= headers, params=params)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    return text

#从列表页面抽取信息存入数据库
def get_cpa_info(token):
    page_num = get_cpa_num(token)
    if page_num==0:
        return 0
    profile_tokens = []
    for page in range(1, page_num+1):
        text = get_cpa_page(token, page)
        soup = BeautifulSoup(text, 'lxml')
        td = soup.find_all('table', "detail_table")[1].find_all('tr')
        if len(td) < 2:
            return 0
        unit = soup.find('table', "detail_table").find_all('tr')[0].td.get_text('\n', '<br/>').replace('\n','').replace('\t', '').replace('（注册会计师）', '')
        tdl = td[0].find_all('th')
        trcs = td[1:]
        for trc in trcs:
            tdc = trc.find_all('td')
            cpa_piece = {tdl[i].get_text('\n', '<br/>').replace('\n','').replace('\t', ''): tdc[i].get_text('\n', '<br/>').replace('\n','').replace('\t', '') for i in range(len(tdl))}
            cpa_piece['所在事务所'] = unit
            cpa.insert_one(cpa_piece)
        for tk in re.findall('getPerDetails\(\'(.*?)\',', text):
            profile_tokens.append(tk)
    return profile_tokens


#获取注册会计师个人简介
#获取个人简介页面
def get_profile_page(tk):
    url = 'http://cmispub.cicpa.org.cn/cicpa2_web/07/'+ tk + '.shtml'
    r = requests.post(url, headers= headers)
    try:
        #尝试用GB2312解码
        text = r.content.decode('gb2312')
    except:
        #解码失败则尝试用GBK解码
        text = r.content.decode('gbk')
    return text

#从页面抽取信息
def get_one_profile(tk):
    text = get_profile_page(tk)
    soup = BeautifulSoup(text, 'lxml')
    td = soup.find('table', "detail_table")
    tdl = td.find_all('td', "tdl")
    tdc = td.find_all('td', "data_tb_content")
    profile_piece = {tdl[i].get_text('\n', '<br/>').replace('\n','').replace('\t', ''): tdc[i].get_text('\n', '<br/>').replace('\n','').replace('\t', '') for i in range(len(tdl))}
    profile.insert_one(profile_piece)

#获取事务所所有的注册会计师的个人信息    
def get_profile(token):
    tks = get_cpa_info(token)
    if tks == 0:
        return
    for tk in tks:
        get_one_profile(tk)

def work(token):
    try:
        logging.debug('Processing TOKEN: {}'.format(token))
        get_unit_info(token)
        logging.debug('TOKEN: {} Unit Info Downloaded.'.format(token))
        logging.debug('TOKEN: {} Downloading NwPer Info.'.format(token))
        get_NwPer_info(token)
        logging.debug('TOKEN: {} NwPer Info Downloaded.'.format(token))
        logging.debug('TOKEN: {} Downloading CPA Info.'.format(token))
        get_profile(token)
        logging.debug('TOKEN: {} CPA Info Downloaded.'.format(token))
    except:
        #也许有网络错误的时候，再试一次
        time.sleep(5)
        try:
            logging.debug('Processing TOKEN: {}'.format(token))
            get_unit_info(token)
            logging.debug('TOKEN: {} Unit Info Downloaded.'.format(token))
            logging.debug('TOKEN: {} Downloading NwPer Info.'.format(token))
            get_NwPer_info(token)
            logging.debug('TOKEN: {} NwPer Info Downloaded.'.format(token))
            logging.debug('TOKEN: {} Downloading CPA Info.'.format(token))
            get_profile(token)
            logging.debug('TOKEN: {} CPA Info Downloaded.'.format(token))
        except:
            logging.debug('TOKEN: {}  Download Failed!.'.format(token))
            failed.insert_one({'token': token})


token = []
for i in index.find():
    token.append(i['token'])

p = mp.Pool()
p.map_async(work, token)
p.close()
p.join()